<?php

// ---------------------------------------------------------------------------------------------------------------------
// This file contains functions about Crontab Job.
// ---------------------------------------------------------------------------------------------------------------------

// Cronjob time -----------------------------------------------------------------------------------

/* Returns the string of a sequals from 0 to $End by $Step */
function getSequence($End, $Step, $Start = 0) {
	if($Step == 0)    return "0";
	if($Step == 1)    return "*";
	if($Step >= $End) return "0";
	$Seq = "".$Start;
	for($i = ($Start + $Step); $i < $End; $i+=$Step) $Seq .= ",".$i;
	return $Seq;
}

/**
 * Returns the Cronjob time string from the  string
 *
 *      A day in a week:
 *          (sun|mon|tue|wed|thu|fri|sat)
 *          Examples:
 *              sun = backup every sunday (mid night of sunday)
 * 
 *      Every certail amount of time or 'every-x' time:
 *          <Step><Unit> as [0-9]+[mhDM]
 *          For Unit,
 *     	        m = Minute
 *     	        h = Hour
 *     	        D = Day
 *     	        M = Month
 *          Examples:
 *              10m = every 10 minutes
 *              12h = every 12 hours
 *              14D = every 14 days
 *
 *      A valid cronjob time:
 *          "0 0 1 * *" = For at the 1st midnight of every month
 */
function getCronjobTimeForEvery($Time) {
	$DayOfWeek = array("sun" => 0, "mon" => 1, "tue" => 2, "wed" => 3, "thu" => 4, "fri" => 5, "sat" => 5);

	if(array_key_exists($Time, $DayOfWeek))
		return date("i H")." * * ".$DayOfWeek[$Time];

	else if(ereg("((([0-9]+(\-[0-9]+)?(,[0-9]+(\-[0-9]+)?)*)|\*) +){4}([0-6](\-[0-6])?(,[0-6](\-[0-6])?)*|\*)", $Time))
		return $Time;

	else if(ereg ("([0-9]+)([mhDM]|min|hour|day|mouth)", $Time, $Times)) {		
		$CronT = null;
		$T = $Times[1]; $T = 0+$T;
		$U = $Times[2];

		if(     $U ==   "min") $U = "m";
		else if($U ==  "hour") $U = "h";
		else if($U ==   "day") $U = "D";
		else if($U == "mouth") $U = "M";

		if(     $U == "m") $CronT = ""            .getSequence(60, $T, 0)." * * * *";
		else if($U == "h") $CronT = "0 "            .getSequence(24, $T, 0)." * * *";
		else if($U == "D") $CronT = "* ".date("H")." ".getSequence(31, $T, 1)." * *";
		else if($U == "M") $CronT = "* * ".date("j")." ".getSequence(12, $T, 1)." *";
		return $CronT;
	}
	return null;
}

/**
 * Returns the time for the next operation in a crontab job format.
 * The time produce by this function will ensure that the operation will be done after the specific period of time.
 * By given the crontab schedule of operating every single minute after the time period. It is very important for
 * the operation to remove the schedule and recreate the new one for the next operation.
 */
function getCronjobTimeForNext($Time) {
	if(!ereg ("([0-9]+)([mhDWM]|mins?|hours?|days?|weeks?|mouths?)", $Time, $Times)) return null;

	$Time = $Times[1]; $Time = 0+$Time;
	$Unit = $Times[2];

	if(     strStartWith("min",   $Unit) || ($Unit ==  "m")) { $Unit =   "min";             }
	else if(strStartWith("hour",  $Unit) || ($Unit ==  "h")) { $Unit =  "hour";             }
	else if(strStartWith("day",   $Unit) || ($Unit ==  "D")) { $Unit =   "day";             }
	else if(strStartWith("week",  $Unit) || ($Unit ==  "W")) { $Unit =   "day"; $Time *= 7; }
	else if(strStartWith("mouth", $Unit) || ($Unit ==  "M")) { $Unit = "month";             }

	$Restarts = array( 0,  0, 1, 1);
	$Maxs     = array(59, 23, 0+date("t"), 12);
	$NextMaxs = array(59, 23, 0+date("t", strtotime("+1month")), 12);

	$Starts   = split(" ", date('i G j m Y'));
	$Ends     = split(" ", date('i G j m Y', strtotime('+'.$Time.$Unit)));

	$CronTime      = " ";
	$IsFirst       = true;
	$IsPrevEqual   =  ($Ends[(count($Starts) - 1)] == $Starts[(count($Starts) - 1)]);
	$IsPrevTwoFold = (($Ends[(count($Starts) - 1)] -  $Starts[(count($Starts) - 1)]) >= 2);

	for($i = (count($Starts) - 1); --$i >= 0; ) {
		if($IsPrevTwoFold) $CronTime = "* ".$CronTime;
		else {
			$Diff = $Ends[$i] - $Starts[$i];
		
			if($IsPrevEqual) {
				if($Diff == 0)
					$CronTime = "* ".$CronTime;

				else {
					if($Ends[$i] == $NextMaxs[$i]) $CronTime =  (0+$Ends[$i])                       ." ".$CronTime;
					else                           $CronTime = ((0+$Ends[$i])."-".(0+$NextMaxs[$i]))." ".$CronTime;

					if(     ($Starts[$i] - 1) <  $Restarts[$i]) $CronTime =  "".$CronTime;
					else if(($Starts[$i] - 1) == $Restarts[$i]) $CronTime =  $Restarts[$i]                         .",".$CronTime;
					else                                        $CronTime = ($Restarts[$i]."-".(0+$Starts[$i] - 1)).",".$CronTime;
				}

			} else {
				if($IsFirst) {
					if(     ($Starts[$i] - 1) <= $Ends[$i]) return null;
					else                                    $CronTime = ((0+$Ends[$i])."-".(0+$Starts[$i] - 1))." ".$CronTime;
				} else {
					if(     $Ends[$i] == $Restarts[$i]) $CronTime =  "* ".$CronTime;
					else if($Ends[$i] == $NextMaxs[$i]) $CronTime =  (0+$Ends[$i])                       ." ".$CronTime;
					else                                $CronTime = ((0+$Ends[$i])."-".(0+$NextMaxs[$i]))." ".$CronTime;
				}
			}

			$IsFirst       = false;
			$IsPrevTwoFold = !$IsPrevEqual || ($Diff >= 2);
			$IsPrevEqual   = ($Diff == 0);
		}
	}

	return trim($CronTime)." *";
}

/** Returns the list od CrontabJob currently install */
function listCrontabJob() {
	return shell_Exec('crontab -l');
}

/**
 * Install a Crontab Job.
 * $Command must control no double qoute sign.
 */
function installCrontabJob($CronTime, $Command, $EnableEmailNotification) {
	$EmailNotification = $EnableEmailNotification?' > /dev/null 2>&1':'';
	return trim(shell_Exec('(crontab -l ; echo "'.$CronTime.' '.$Command.' '.$EmailNotification.'") | crontab -'));
}


?>
